/*   ACUA - Access Control and User Administration.
 *   Copyright (C) 2000  Robert Davidson.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *   You may not sell ACUA or any part of ACUA for profit.
 */

#include <assert.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <syslog.h>
#include <errno.h>
#include "common.h"

typedef struct _sList sList;

struct _sList {
		UserRec	ur;
		sList	*nextUserRec;
};

int s_search(uid_t uid, sList *localList)
{
  for(;;) {
		  // End of list
		  if (localList->nextUserRec == NULL)
				  return 1;
		  if (localList->ur.uid == uid)
				  return 0;
		  localList = localList->nextUserRec;
  };
};

int
acua_sync(int argc, char **argv)
{
  UserRec                  ur;
  char                    *tUserPath;
  FILE                    *tUserFile;
  int                      recNum;
  sList			  *userList,
      			  *firstList;

  argc = argc; argv = argv;
  recNum = 0;

  umask(077);

  readConfig();
  userFileOpen();

  // create a tempory file
  if ((tUserPath = (char *) malloc(strlen(userFilePath)+8)) == NULL)
      perrlQuit("acua_sync.cc:malloc(1)");

  sprintf(tUserPath, "%s%s", userFilePath, ".XXXXXX");

  printf("tUserPath: %s\n", tUserPath);

  if (!mkstemp(tUserPath))
      perrlQuit("acua_sync.cc:mktemp(1)");

  if (chmod(tUserPath, 0644))
      perrlQuit("acua_sync.cc:chmod(1)");

  if ((tUserFile = fopen(tUserPath, "w+b")) == NULL)
      perrlQuit("acua_sync.cc:fopen(1)");

  fwrite(ACUA_USERS_MAGIC, 1, ACUA_USERS_MAGIC_LEN, tUserFile);

//  syslog(LOG_ERR, "About to run Aleskey's code in acua_sync.cc");

  // FIXME: this part could do with some optimization i think.

  // - TEST ME PLEASE

  // segmentation fault probably being caused when s_search() is called,
  // since it could be searching a memory range thats not been written
  // to, only malloc'd.

  // Disadventage - always last item in list don`t using 8-)
  if (!(firstList = userList = (sList *)malloc(sizeof(sList))))
      perrlQuit("acua_sync.cc:malloc(2)");

  while (!userFileRead(&ur)) {

      if (getpwuid(ur.uid) == NULL)
          if (errno == ENOMEM)
              perrlQuit("acua_sync.cc:getpwuid(1)");
                  else continue;

      memcpy(&userList->ur, &ur, sizeof(UserRec));
      userList->nextUserRec = (sList *)malloc(sizeof(sList));

      if (userList->nextUserRec == NULL)
          perrlQuit("acua_sync.cc:malloc(3)");

      userList = userList->nextUserRec;
      userList->nextUserRec = NULL;
  };

  userList = firstList;	// Go to begin list
  for(;;) {
		  // End of list
		  if (userList->nextUserRec == NULL)
				  break;
		  firstList = userList;
		  fwrite(&userList->ur, sizeof(UserRec), 1, tUserFile);
		  userList = userList->nextUserRec;
		  free(firstList);
  };

  // - END TEST ME PLEASE

//  syslog(LOG_ERR, "End of Aleskey's code");

  userFileClose();

  if (fclose(tUserFile)) {
    // error writing to our tempory file.
    unlink(tUserPath);
    free(tUserPath);
    errlQuit("acua_sync.cc:fclose(tUserFile) - failed");
  }

  if (rename(tUserPath, userFilePath)) {
    // error moving the new database over the old one.
    unlink(tUserPath);
    free(tUserPath);
    errlQuit("acua_sync.cc:rename(1)");
  }

  // unlink(tUserPath);
  free(tUserPath);
  return(0);
}
